unit Mdraw;

interface

uses
  StdCtrls, SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Buttons, RXCombos, Spin, Menus;

type
  TDraws=(DrawText,DrawLine, DrawRectangle, DrawEllipse, DrawCircle, DrawKvadrat);

  TMDraw = class(TPanel)
  private
    { Private declarations }
    MBtnFont:TSpeedButton;
    MDraws:TDraws;
    MImgSB:TScrollBox;
    MBtnPanel:TScrollBox;
    MStatus:TPanel;
    MBtnLineCol:TColorComboBox;
    MBtnFillCol:TColorComboBox;
    MLineWidth:TSpinEdit;
    MTool:TComboBox;
    MFontColor:TColorComboBox;
    FontDlg:TFontDialog;
    FImageWidth:Integer;
    FImageheight:Integer;
    FLineColor:TColor;
    FFillColor:TColor;
    FLineWidth:Integer;
    MImage:TImage;
    MText:TEdit;

    Drawing:Boolean;

    Procedure SetLineColor(C:TColor);
    Procedure SetFillColor(C:TColor);
    Procedure SetLineWidth(C:Integer);
    Procedure SetImageWidth(C:Integer);
    Procedure SetImageHeight(C:Integer);
    procedure SetFontColor(c:TColor);
    Procedure LineColorChange(Sender:TObject);
    Procedure FillColorChange(Sender:TObject);
    Procedure LineWidthChange(Sender:TObject);
    Procedure ToolChange(Sender:TObject);
    Procedure FontChange(Sender:TObject);
    Procedure FontColorChange(Sender:TObject);
    Procedure FontClick(Sender:TObject);
    procedure StartDraw(Sender: TObject; Button: TMouseButton;
                Shift: TShiftState; X, Y: Integer);
    procedure EndDraw(Sender: TObject; Button: TMouseButton;
                Shift: TShiftState; X, Y: Integer);
    procedure DrawMe(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    Procedure DrawShape(Style:TPenMode);
  protected
    { Protected declarations }
  public
    { Public declarations }
    First,Last:TPoint;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure OnResize; dynamic;
    Procedure LoadPicture(S:String);
    Procedure SavePicture(S:String);
  published
    { Published declarations }
    property LineColor:TColor read FLineColor write SetLineColor default clBlack;
    property LineWidth:integer read FLineWidth write SetLineWidth default 1;
    property fillColor:TColor read FFillColor write SetFillColor default clWhite;
    property ImageWidth:integer read FImageWidth write SetImageWidth default 150;
    property ImageHeight:Integer read FImageHeight write SetImageHeight default 100;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MIKS', [TMDraw]);
end;

constructor TMDraw.Create(AOwner: TComponent);
var Newitem:TMenuItem;
    i:integer;
begin
  inherited Create(AOwner);
  Parent := TWinControl(AOwner);
  BorderStyle:=bsSingle;
  Ctl3d:=False;
  BevelInner:=bvLowered;
  BorderWidth:=2;
  MDraws:=DrawLine;
  Drawing:=False;
  First:=Point(0,0);
  Last:=Point(0,0);
  ShowHint:=True;
  FImageWidth:=150;
  FImageHeight:=150;

  FontDlg:=TFontDialog.CreaTE(sELF);

  MBtnPanel:=TScrollBox.Create(Self);
  With MBtnPanel do begin
    Parent:=Self;
    bevelInner:=bvLowered;
    Align:=AlTop;
    Height:=25;
    Hint:='Opcije';
  end;

  MImgSB:=TScrollBox.create(Self);
  with MImgSB do begin
    Parent:=Self;
    Align:=AlClient;
    BorderWidth:=2;
    bevelOuter:=bvNone;
    BorderStyle:=bssingle;
  end;

  MStatus:=TPanel.Create(Self);
  With MStatus do begin
    Parent:=Self;
    bevelOuter:=bvLowered;
    Alignment:=taLeftJustify;
    Align:=AlBottom;
    Height:=20;
    Hint:='Status';
  end;

  MImage:=TImage.Create(self);
  with MImage do begin
    Parent:=MImgSB;
    Align:=AlNone;
    OnMouseDown:=StartDraw;
    OnMouseUp:=EndDraw;
    OnMouseMove:=DrawMe;
    Stretch:=False;
    Picture:=TPicture.Create;
    Width:=200;
    Height:=200;
  end;


  FLineColor:=ClBlack;
  MBtnLineCol:=TColorComboBox.Create(Self);
  With MBtnLineCol do begin
    DisplayNames:=False;
    Color:=ClWindow;
    Parent:=MBtnPanel;
    Align:=AlLeft;
    Width:=50;
    ColorValue:=FLineColor;
    Hint:='Boja linije';
    OnChange:=LineColorChange;
  end;

  FFillColor:=ClWhite;
  MBtnFillCol:=TColorComboBox.Create(Self);
  With MBtnFillCol do begin
    Parent:=MBtnPanel;
    DisplayNames:=False;
    Color:=ClWindow;
    Align:=AlLeft;
    Width:=50;
    ColorValue:=FFillColor;
    hint:='Boja izpunjavanja';
    OnChange:=FillColorChange;
  end;

  MFontColor:=TColorComboBox.Create(Self);
  With MFontColor do begin
    Parent:=MBtnPanel;
    DisplayNames:=False;
    Colorvalue:=clBlack;
    Align:=AlLeft;
    Width:=50;
    hint:='Boja teksta';
    OnChange:=FontColorChange;
  end;

  MLineWidth:=TSpinEdit.Create(self);
  With MLineWidth do begin
    parent:=MBtnPanel;
    Hint:='Debljina linije';
    Value:=1;
    Align:=AlLeft;
    Width:=40;
    OnChange:=LineWidthChange;
  end;

  MTool:=TComboBox.Create(Self);
  With MTool do begin
    Parent:=MBtnPanel;
    Width:=130;
    Align:=AlLeft;
    Sorted:=True;
    Clear;
    Items.Add('Linija');
    Items.Add('Tekst');
    Items.Add('Pravougaonik');
    Items.Add('Kvadrat');
    Items.Add('Elipsa');
    Items.Add('Krug');
    OnChange:=ToolChange;
  end;

  mbTNfONT:=TSpeedButton.Create(Self);
  with mbTNfONT do begin
    OnClick:=FontClick;
    parent:=MBtnPanel;
    Align:=AlLeft;
    Caption:='Fnt';
    Hint:='Font';
  end;

  MText:=TEdit.Create(Self);
  With MText do begin
    parent:=MBtnPanel;
    Align:=AlClient;
    Text:='';
    Hint:='Tekst';
  end;
end;

destructor TMDraw.Destroy;
begin
  inherited Destroy;
end;

Procedure TMDraw.FontClick;
begin
  With FontDlg do if execute then MImage.Canvas.Font:=Font;
end;

procedure TMDraw.StartDraw(Sender: TObject; Button: TMouseButton;
                Shift: TShiftState; X, Y: Integer);
begin
  Drawing:=True;
  First:=Point(x,y);
  Last:=First;
  MImage.Canvas.MoveTo(x,y);
  MStatus.Caption:=Format('Pozicija (%d,%d)',[x,y]);
end;

procedure TMDraw.EndDraw(Sender: TObject; Button: TMouseButton;
                Shift: TShiftState; X, Y: Integer);
begin
  if drawing then begin
    Last:=Point(x,y);
    DrawShape(pmCopy);
    Drawing:=False;
  end;
  MStatus.Caption:=Format('Pozicija (%d,%d)',[x,y]);
end;

procedure TMDraw.DrawMe(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if Drawing then
  begin
    DrawShape(pmNotXor);
    Last := Point(X, Y);
    DrawShape(pmNotXor);
  end;
  MStatus.Caption:=Format('Pozicija (%d,%d)',[x,y]);
end;

Procedure TMDraw.DrawShape(Style:TPenMode);
var o1,o2,o3:TPoint;
begin
  o1:=First;
  o2:=Last;
  if (First.x>last.x) or (First.y>last.y) then begin
    o3:=last;
    last:=First;
    First:=o3;
  end;
  if drawing then with MImage.Canvas do begin
    Pen.Mode:=Style;
    MoveTo(First.X,First.Y);
    case MDraws of
      DrawLine:LineTo(Last.X,Last.Y);
      DrawRectangle:Rectangle(First.X,First.Y,Last.X,Last.Y);
      DrawKvadrat:Rectangle(First.X,First.Y,Last.x,Last.x);
      DrawCircle:Ellipse(First.X,First.Y,Last.X,Last.X);
      DrawEllipse:Ellipse(First.X,First.Y,Last.X,Last.Y);
      DrawText:TextOut(First.X,First.Y,MText.Text);
    end;
  end;
  First:=o1;
  Last:=o2;
end;

Procedure TMDraw.LineColorChange(Sender:TObject);
begin
  SetLineColor(MBtnLineCol.ColorValue);
end;

Procedure TMDraw.FillColorChange(Sender:TObject);
begin
  SetFillColor(MBtnFillCol.ColorValue);
end;

Procedure TMDraw.LineWidthChange(Sender:TObject);
begin
  SetLineWidth(MLineWidth.Value);
end;

Procedure TMDraw.SetLineColor;
begin
  FLineColor:=C;
  MImage.Canvas.Pen.Color:=FLineColor;
end;

Procedure TMDraw.SetFillColor;
begin
  FFillColor:=C;
  MImage.Canvas.Brush.Color:=FFillColor;
end;

Procedure TMDraw.SetLineWidth;
begin
  FLineWidth:=C;
  MImage.Canvas.Pen.Width:=c;
end;

Procedure TMDraw.SetImageWidth(C:Integer);
begin
  FImageWidth:=C;
  MImage.Width:=C;
end;

Procedure TMDraw.SetFontColor(C:TColor);
begin
  MFontColor.ColorValue:=c;
  FontDlg.Font.Color:=C;
  MImage.Canvas.Font.Color:=c;
end;


Procedure TMDraw.SetImageHeight(C:Integer);
begin
  MImage.Height:=C;
  FImageHeight:=C;
end;

Procedure TMDraw.OnResize;
begin
  With MImage do begin
    Width:=FImageWidth;
    Height:=FImageHeight;
  end;
  invalidate;
  inherited OnResize(Self);
end;

Procedure TMDraw.ToolChange;
begin
  if UpperCase(MTool.Text)='ELIPSA' then MDraws:=DrawEllipse;
  if UpperCase(MTool.Text)='PRAVOUGAONIK' then MDraws:=DrawRectangle;
  if UpperCase(MTool.Text)='KVADRAT' then MDraws:=DrawKvadrat;
  if UpperCase(MTool.Text)='LINIJA' then MDraws:=DrawLine;
  if UpperCase(MTool.Text)='KRUG' then MDraws:=DrawCircle;
  if UpperCase(MTool.Text)='TEKST' then MDraws:=DrawText;
end;

Procedure TMDraw.LoadPicture(S:String);
begin
  if FileSearch(s,'.')<>'' then begin
    With MImage do begin
      Picture.LoadFromFile(S);
      Width:=Picture.Width;
      Height:=Picture.Height;
    end;
  end;
end;

Procedure TMDraw.SavePicture(S:String);
begin
  MImage.Picture.SaveToFile(S);
end;

Procedure TMDraw.FontChange(Sender:TObject);
begin
  MImage.Canvas.Font:=FontDlg.Font;
  MFontColor.ColorValue:=FontDlg.Font.Color;
end;

Procedure TMDraw.FontColorChange(Sender:TObject);
begin
  MImage.Canvas.Font.Color:=MFontColor.ColorValue;
  FontDlg.Font.Color:=MFontColor.ColorValue;
end;


end.


